@@ -1,5 +1,5 @@
 from pyonir.models.app import BaseApp, BasePlugin
-from pyonir.models.database import BaseCollection
+from pyonir.models.database import BaseFSQuery
 from pyonir.models.schemas import BaseSchema
 from pyonir.models.server import BaseRequest, BaseServer
 
@@ -7,6 +7,6 @@ from pyonir.models.server import BaseRequest, BaseServer
 class PyonirApp(BaseApp):pass
 class PyonirServer(BaseServer): pass
 class PyonirRequest(BaseRequest): pass
-class PyonirCollection(BaseCollection): pass
+class PyonirCollection(BaseFSQuery): pass
 class PyonirSchema(BaseSchema): pass
 class PyonirPlugin(BasePlugin): pass
\ No newline at end of file
