@@ -111,11 +111,6 @@ class Base:
         """Directory path for jinja template filters"""
         return os.path.join(self.backend_dirpath, self.FILTERS_DIRNAME)
 
-    @property
-    def datastore_dirpath(self) -> str:
-        """Directory path for datastorage"""
-        return os.path.join(self.app_dirpath, self.DATA_DIRNAME)
-
     @property
     def frontend_dirpath(self) -> str:
         """Directory path for site's theme folders"""
@@ -155,21 +150,22 @@ class Base:
     # RUNTIME
     def process_configs(self):
         """Processes all context settings"""
-        from pyonir.utilities import process_contents
+        from pyonir.models.utils import process_contents
         self._settings = process_contents(self.configs_dirpath, app_ctx=self.app_ctx)
 
-    def parse_file(self, file_path: str) -> 'Parsely':
+    def parse_file(self, file_path: str) -> 'DeserializeFile':
         """Parses a file and returns a Parsely instance for the file."""
-        from pyonir.parser import Parsely
-        return Parsely(file_path, app_ctx=self.app_ctx)
+        # from pyonir.parser import Parsely
+        from pyonir.models.page import DeserializeFile
+        return DeserializeFile(file_path, app_ctx=self.app_ctx)
 
     def apply_virtual_routes(self, pyonir_request: 'BaseRequest') -> 'Parsely':
         """Reads and applies virtual .routes.md file specs onto or updates the request file"""
-        from pyonir.parser import ParselyFileStatus, Parsely
+        from pyonir.models.parser import DeserializeFile, FileStatuses
         server = pyonir_request.app.server
         virtual_route_url, virtual_route_data, virtual_path_params = server.get_virtual(pyonir_request.path)
         if virtual_route_data:
-            rfile: Parsely = pyonir_request.file
+            rfile: DeserializeFile = pyonir_request.file
             pyonir_request.path_params.update(virtual_path_params)
             if rfile.file_exists:
                 rfile.data.update(virtual_route_data)
@@ -181,9 +177,9 @@ class Base:
                 vurl_data = vfile.data.get(virtual_route_url) or {}
                 vurl_data.update(**{'url': pyonir_request.path, 'slug': pyonir_request.slug})
                 vfile.data = vurl_data
-                vfile.status = ParselyFileStatus.PUBLIC
-                vfile.file_ssg_html_dirpath = vfile.file_ssg_html_dirpath.replace(vfile.file_name, pyonir_request.slug)
-                vfile.file_ssg_api_dirpath = vfile.file_ssg_api_dirpath.replace(vfile.file_name, pyonir_request.slug)
+                vfile.status = FileStatuses.PUBLIC
+                # vfile.file_ssg_html_dirpath = vfile.file_ssg_html_dirpath.replace(vfile.file_name, pyonir_request.slug)
+                # vfile.file_ssg_api_dirpath = vfile.file_ssg_api_dirpath.replace(vfile.file_name, pyonir_request.slug)
                 pyonir_request.file = vfile
 
     def register_resolver(self, name: str, cls_or_path, args=(), kwargs=None, hot_reload=False):
@@ -216,7 +212,7 @@ class Base:
         """
         import importlib, sys
         from pyonir.utilities import get_attr
-        from pyonir.parser import Parsely
+        from pyonir.models.loaders import load_resolver
 
         cls_path, meth_name = name.rsplit(".", 1)
         is_pyonir = name.startswith('pyonir')
@@ -241,7 +237,7 @@ class Base:
         # access modules from loader
         if not resolver:
             from pyonir import PYONIR_DIRPATH
-            resolver = Parsely.load_resolver(name,
+            resolver = load_resolver(name,
                                           base_path=PYONIR_DIRPATH if is_pyonir else self.app_dirpath,
                                           from_system=is_pyonir)
         if not resolver:
@@ -392,8 +388,9 @@ class BaseApp(Base):
     def __init__(self, app_entrypoint: str, use_themes: bool = None):
         from pyonir.models.templating import TemplateEnvironment
         from pyonir.core import PyonirServer
-        from pyonir.parser import parse_markdown
+        from pyonir.models.parser import parse_markdown, DeserializeFile
         from pyonir import __version__
+        DeserializeFile._routes_dirname = self.PAGES_DIRNAME
         self.VERSION = __version__
         self.SECRET_SAUCE = generate_id()
         self.SESSION_KEY = f"{self.name}_session"
@@ -425,7 +422,7 @@ class BaseApp(Base):
     def env(self) -> EnvConfig: return self._env
 
     @property
-    def use_ssl(self) -> str: return get_attr(self.env, 'app.use_ssl')
+    def use_ssl(self) -> bool: return get_attr(self.env, 'app.use_ssl',  '').strip().lower() in ('true', '1', 'yes')
 
     @property
     def salt(self) -> str: return get_attr(self.env, 'app.salt')
@@ -474,14 +471,28 @@ class BaseApp(Base):
 
     @property
     def nginx_config_filepath(self):
-        return os.path.join(self.app_dirpath, self.name + '.conf')
+        default = os.path.join(self.app_dirpath, self.name + '.conf')
+        return get_attr(self.env, 'app.nginx_conf_dirpath') or default
 
     @property
     def unix_socket_filepath(self):
         """WSGI socket file reference"""
-        return os.path.join(self.app_dirpath, self.name+'.sock')
+        default = os.path.join(self.app_dirpath, self.name+'.sock')
+        return get_attr(self.env, 'app.unix_socket_dirpath') or default
 
     # DIRECTORIES
+    @property
+    def app_account_dirpath(self) -> str:
+        """Parent directory location for applications"""
+        return os.path.dirname(self.app_dirpath)
+
+    @property
+    def datastore_dirpath(self) -> str:
+        """Directory path for file system data storage is one level above the application directory
+        and labeled as {appname}_data_stores"""
+        default = os.path.join(self.app_account_dirpath, f"{self.name}_{self.DATA_DIRNAME}")
+        return get_attr(self.env, 'app.data_dirpath') or default
+
     @property
     def frontend_assets_dirpath(self) -> str:
         """Directory location for template related assets"""
@@ -606,10 +617,11 @@ class BaseApp(Base):
 
     def parse_pyformat(self, string, context=None) -> str:
         """Formats python template string"""
-        ctx = self.TemplateEnvironment.globals if self.TemplateEnvironment else {}
+        if not context: context = {}
+        if self.TemplateEnvironment:
+            context.update(self.TemplateEnvironment.globals)
         try:
-            if context is not None: ctx.update(context)
-            return string.format(**ctx)
+            return string.format(**context)
         except Exception as e:
             print('parse_pyformat', e, string)
             return string
@@ -636,7 +648,8 @@ class BaseApp(Base):
         import time
         from pyonir import utilities
         from pyonir.models.server import BaseRequest
-        from pyonir.parser import Parsely
+        from pyonir.models.parser import DeserializeFile
+        from pyonir.models.database import query_fs
         self.SSG_IN_PROGRESS = True
         count = 0
         print(f"{utilities.PrntColrs.OKBLUE}1. Coping Assets")
@@ -651,13 +664,16 @@ class BaseApp(Base):
             ssg_req = BaseRequest(None, self)
             start_time = time.perf_counter()
 
-            all_pages: Generator[Parsely] = utilities.query_files(self.pages_dirpath, app_ctx=self.app_ctx, model='parsely')
+            all_pages: Generator[DeserializeFile] = query_fs(self.pages_dirpath, app_ctx=self.app_ctx, model='file')
             xmls = []
             for pgfile in all_pages:
-                virtual_data, path_params = self.server.get_virtual(pgfile.data.get('url'))
+                virtual_data, path_params, *rest = self.server.get_virtual(pgfile.data.get('url'))
                 if path_params: ssg_req.ssg_request(pgfile, path_params)
-                pgfile.data.update(virtual_data)
-                pgfile.apply_filters()
+                try:
+                    pgfile.data.update(path_params or {})
+                    pgfile.apply_filters()
+                except Exception as e:
+                    raise
                 self.TemplateEnvironment.globals['request'] = ssg_req  # pg_req
                 count += pgfile.generate_static_file()
                 t = f"<url><loc>{self.protocol}://{self.domain}{pgfile.data.get('url')}</loc><priority>1.0</priority></url>\n"
